package net.nigreon.blegps

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.bluetooth.BluetoothGatt
import android.content.Intent
import android.os.Binder
import android.os.Build
import android.os.IBinder
import android.support.v4.app.NotificationCompat
import android.support.v4.content.LocalBroadcastManager
import android.util.Log
import com.clj.fastble.BleManager
import com.clj.fastble.callback.BleGattCallback
import com.clj.fastble.callback.BleNotifyCallback
import com.clj.fastble.callback.BleWriteCallback
import com.clj.fastble.data.BleDevice
import com.clj.fastble.exception.BleException
import java.nio.ByteBuffer
import java.nio.ByteOrder
import java.util.*

class BLEHRService  : Service() {
    private val LOG_TAG = "BLEHRService"
    private val CHANNEL_ID = "NotifID2"
    private val binder = sBinder()

    val timerActivity = Timer()

    private lateinit var bleDevice: BleDevice
    private val serviceHeartRateUuid: String = "0000180d-0000-1000-8000-00805f9b34fb"
    private val characteristicHRControlPointUuid: String = "00002a39-0000-1000-8000-00805f9b34fb"
    private val characteristicHRMeasurementUuid: String = "00002a37-0000-1000-8000-00805f9b34fb"

    private fun initNotificationHR() {
        BleManager.getInstance().notify(
            bleDevice,
            serviceHeartRateUuid,
            characteristicHRMeasurementUuid,
            object : BleNotifyCallback() {
                override fun onNotifySuccess() {
                    Log.v(LOG_TAG, "Notify HR Success")
                    //initNotificationPressure()
                }

                override fun onNotifyFailure(exception: BleException) {
                    Log.v(LOG_TAG, "BLE Notify HR Failure $exception")
                    //initNotificationPressure()
                }

                override fun onCharacteristicChanged(data: ByteArray) {
                    //Log.v(LOG_TAG, "Notify Temperature Characteristic changed")
                    val buffer = ByteBuffer.wrap(data)
                    buffer.order(ByteOrder.LITTLE_ENDIAN)
                    //texttemp.text = buffer.float.toString()
                    LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                        Intent(ServicesConstants.BROADCAST_FILTER.FILTERBTHRM).putExtra(ServicesConstants.BROADCAST_KEY.KEYBTHRM, buffer.get(1))
                    )
                }
            })
    }

    private fun startHRBLE() {
        BleManager.getInstance().connect("CC:2C:24:71:9C:BC", object : BleGattCallback() {
            override fun onStartConnect() {

            }

            override fun onConnectFail(bleDevice: BleDevice, exception: BleException) {
                Log.v(LOG_TAG, "BLE HR Connection Failed $exception")
            }

            override fun onConnectSuccess(
                bleDeviceConnect: BleDevice,
                gattConnect: BluetoothGatt,
                status: Int
            ) {
                Log.v(LOG_TAG, "BLE HR Connection OK")
                LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                    Intent(ServicesConstants.BROADCAST_FILTER.FILTERBTHR).putExtra(
                        ServicesConstants.BROADCAST_KEY.KEYBTSTATUSHR,
                        true
                    )
                )
                bleDevice = bleDeviceConnect

                initNotificationHR()

                val t = object : TimerTask() {
                    override fun run() {
                        startHRBLEActivity()
                    }
                }
                timerActivity.scheduleAtFixedRate(t, 0, 9500)

            }

            override fun onDisConnected(
                isActiveDisConnected: Boolean,
                bleDevice: BleDevice,
                gatt: BluetoothGatt,
                status: Int
            ) {
                LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                    Intent(ServicesConstants.BROADCAST_FILTER.FILTERBTHR).putExtra(
                        ServicesConstants.BROADCAST_KEY.KEYBTSTATUSHR,
                        false
                    )
                )
                timerActivity.cancel()
                Log.v(LOG_TAG, "BLE HR Disconnected")
            }
        })
    }

    private fun stopHRBLE() {
        LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
            Intent(ServicesConstants.BROADCAST_FILTER.FILTERBTHR).putExtra(ServicesConstants.BROADCAST_KEY.KEYBTSTATUSHR, false)
        )

        BleManager.getInstance().disconnect(bleDevice)
    }

    private fun startHRBLEActivity() {
        val data: ByteArray = byteArrayOf(0x15,0x01,0x01)
        BleManager.getInstance().write(
            bleDevice,
            serviceHeartRateUuid,
            characteristicHRControlPointUuid,
            data,
            object : BleWriteCallback() {
                override fun onWriteSuccess(current: Int, total: Int, justWrite: ByteArray) {
                    //Log.v(LOG_TAG, "Start Activity sended")
                }

                override fun onWriteFailure(exception: BleException) {
                    Log.v(LOG_TAG, "Start Activity Failure $exception")
                }
            })
    }

    private fun stopHRBLEActivity() {
        val data: ByteArray = byteArrayOf(0x15,0x01,0x00)
        BleManager.getInstance().write(
            bleDevice,
            serviceHeartRateUuid,
            characteristicHRControlPointUuid,
            data,
            object : BleWriteCallback() {
                override fun onWriteSuccess(current: Int, total: Int, justWrite: ByteArray) {
                    Log.v(LOG_TAG, "Stop Activity sended")
                }

                override fun onWriteFailure(exception: BleException) {
                    Log.v(LOG_TAG, "Stop Activity send Failure $exception")
                }
            })
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        if(intent.action == ServicesConstants.HRSERVICEACTION.STARTFOREGROUND_ACTION) {
            Log.i(LOG_TAG, "Received Start Foreground Intent ")

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                // Create the NotificationChannel
                val name = "ChannelName"
                val descriptionText = "ChannelDescription"
                val importance = NotificationManager.IMPORTANCE_DEFAULT
                val mChannel = NotificationChannel(CHANNEL_ID, name, importance)
                mChannel.description = descriptionText
                // Register the channel with the system; you can't change the importance
                // or other notification behaviors after this
                val notificationManager = getSystemService(NOTIFICATION_SERVICE) as NotificationManager
                notificationManager.createNotificationChannel(mChannel)
            }

            var notification = NotificationCompat.Builder(this, CHANNEL_ID)
                .setSmallIcon(R.drawable.ic_launcher_foreground)
                .setContentTitle("BLE HR")
                .setContentText("BLE HR active")
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                //.setContentIntent(pendingIntent)
                .build()

            //pBLE = BLEProvider(application, baseContext)
            startHRBLE()

            startForeground(ServicesConstants.NOTIFICATION_ID.FOREGROUND_HR_SERVICE,notification)
        } else if(intent.action == ServicesConstants.HRSERVICEACTION.STOPFOREGROUND_ACTION) {
            Log.i(LOG_TAG, "Received Stop Foreground Intent ")
            //pBLE.disconnectBLE()
            stopHRBLE()
            stopForeground(true)
            stopSelf()
        }
        return START_STICKY
    }

    override fun onDestroy() {
        super.onDestroy()
        Log.i(LOG_TAG, "In onDestroy")
    }

    override fun onBind(intent: Intent): IBinder? {
        Log.v(LOG_TAG, "in onBind")
        return binder
    }

    override fun onRebind(intent: Intent) {
        Log.v(LOG_TAG, "in onRebind")
        super.onRebind(intent)
    }

    override fun onUnbind(intent: Intent): Boolean {
        Log.v(LOG_TAG, "in onUnbind")
        return true
    }

    inner class sBinder : Binder() {
        // Return this instance of LocalService so clients can call public methods
        fun getService(): BLEHRService = this@BLEHRService
    }
}