package net.nigreon.blegps

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.location.LocationManager
import android.os.*
import android.support.v4.app.NotificationCompat
import android.support.v4.content.LocalBroadcastManager
import android.util.Log
import java.io.File
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException
import java.nio.ByteBuffer
import java.nio.ByteOrder
import java.text.SimpleDateFormat
import java.util.*
import kotlin.experimental.or


class BLEMockLocationService : Service() {
    private val LOG_TAG = "BLEMockLocationService"
    private val CHANNEL_ID = "NotifID"
    public val ACTIVATION_CHANGE = 1
    private lateinit var pBLE: BLEProvider
    private lateinit var mockGPS: MockLocationProvider
    private var locationReceived = false
    private var qualityReceived = false
    private var gpsverbose = false
    private var gpxopened = false
    private var hrenable = false
    private var tempenable = false
    private lateinit var gpxoutputstream: FileOutputStream

    var gpsConf: GPSConfiguration = GPSConfiguration()

    // Location
    private var currentLatitude: Double = -1.0
    private var currentLongitude: Double = -1.0
    private var currentElevation: Double = -1.0
    private var currentSpeed: Float = -1.0f
    private var currentHeading: Float = -1.0f
    private var currentEHPE: Float = -1.0f
    // Quality
    private var currentEVPE: Float = -1.0f
    private var currentHDOP: Float = -1.0f
    private var currentVDOP: Float = -1.0f
    private var currentSatview: Byte = -1
    private var currentSatused: Byte = -1
    private var currentElevationGPS: Double = -1.0

    private var currentHR: Byte = 0
    private var currentTemperature: Float = 0.0f

    private val binder = sBinder()

    fun locationChanged(intent: Intent)
    {
        Log.v(LOG_TAG, "Latitude Service Broadcast received")
        currentLatitude = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLAT,-1.0)
        currentLongitude = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLON,-1.0)
        currentEHPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONEHPE,-1.0f)
        currentElevation = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONELE,-1.0)
        currentSpeed = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONSPEED,-1.0f)
        currentHeading = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONHEADING,-1.0f)
        mockGPS.pushLocation(currentLatitude,currentLongitude,currentElevation, currentSpeed, currentHeading, currentEHPE)
        locationReceived = true
        storeGPX()
    }

    fun qualityChanged(intent: Intent)
    {
        //Log.v(LOG_TAG, "Latitude Service Broadcast received")
        currentEHPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEHPE,-1.0f)
        currentEVPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEVPE,-1.0f)
        currentHDOP = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYHDOP,-1.0f)
        currentVDOP = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYVDOP,-1.0f)
        currentSatview = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYSATVIEW,-1)
        currentSatused = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYSATUSED,-1)
        currentElevationGPS = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYELEGPS,-1.0)
        qualityReceived = true
        storeGPX()
    }

    fun HRChanged(intent: Intent)
    {
        currentHR = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYBTHRM,-1)
    }

    fun bmpStatusChanged(intent: Intent)
    {
        tempenable = intent.getBooleanExtra(ServicesConstants.BROADCAST_KEY.KEYBMPSTATUS, false)
    }

    fun tempChanged(intent: Intent)
    {
        currentTemperature = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYTEMP, -1.0f)
    }

    fun taskerChanged(bundle: Bundle) {
        Log.v(LOG_TAG, "taskerChanged")
        if(bundle.containsKey(ServicesConstants.BROADCAST_KEY.KEYBOOST)) {
            /*if (bundle.getBoolean(ServicesConstants.BROADCAST_KEY.KEYBOOST)) {
                //gpsConf.gpsBoost = intent.getBooleanExtra(ServicesConstants.BROADCAST_KEY.KEYBOOST, false)
                gpsConf.gpsBoost = true
            } else {
                gpsConf.gpsBoost = false
            }*/
            gpsConf.gpsBoost = bundle.getBoolean(ServicesConstants.BROADCAST_KEY.KEYBOOST)
            sendConfBLE(gpsConf)
            Log.v(LOG_TAG, "Received Tasker Intent : Boost ${gpsConf.gpsBoost}")
        }
    }

    fun storeGPX()
    {
        if(gpxopened) {
            if(!gpsverbose and locationReceived)
            {
                //writeGPX("${SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").format(Date())} $currentLatitude $currentLongitude $currentElevation m\n")
                writeGPX("<trkpt lat=\"$currentLatitude\" lon=\"$currentLongitude\"><ele>$currentElevation</ele><time>${SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").format(Date())}</time></trkpt>\n")
                locationReceived = false
                qualityReceived = false
            } else if(gpsverbose and locationReceived and qualityReceived) {
                //writeGPX("${SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").format(Date())} $currentLatitude $currentLongitude $currentElevation m\n")
                //writeGPX("$currentEHPE m $currentEVPE m $currentHDOP $currentVDOP $currentSatused/$currentSatview $currentElevationGPS m\n")
                //if(hrenable) { writeGPX("$currentHR bpm\n")}
                //if(tempenable) { writeGPX("${currentTemperature}°C\n")}
                writeGPX("<trkpt lat=\"$currentLatitude\" lon=\"$currentLongitude\"><ele>$currentElevation</ele><time>${SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").format(Date())}</time>\n" +
                        "<extensions>\n")
                        if(hrenable or tempenable)
                        {
                            writeGPX("<gpxtpx:TrackPointExtension>")
                            if(hrenable)
                            {
                                writeGPX("<gpxtpx:atemp>$currentHR</gpxtpx:atemp>")
                            }
                            if(tempenable)
                            {
                                writeGPX("<gpxtpx:atemp>$currentTemperature</gpxtpx:atemp>")
                            }
                            writeGPX("</gpxtpx:TrackPointExtension>\n")
                        }

                        writeGPX("<blegps:BLEGPSExtension><blegps:ehpe>$currentEHPE</blegps:ehpe><blegps:evpe>$currentEVPE</blegps:evpe><blegps:hdop>$currentHDOP</blegps:hdop><blegps:vdop>$currentVDOP</blegps:vdop><blegps:satused>$currentSatused</blegps:satused><blegps:satview>$currentSatview</blegps:satview><blegps:gpselevation>$currentElevationGPS</blegps:gpselevation></blegps:BLEGPSExtension>\n")
                        writeGPX("</extensions>\n</trkpt>\n")
                locationReceived = false
                qualityReceived = false
            }
        }
    }

    fun writeGPX(str: String)
    {
        try {
            gpxoutputstream.write(str.toByteArray())
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        } catch (e: IOException) {
            e.printStackTrace()
        }

    }

    override fun onCreate() {
        super.onCreate()

        mockGPS = MockLocationProvider(LocationManager.GPS_PROVIDER, baseContext)

        val brReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context?, intent: Intent?) {
                when (intent?.action) {
                    ServicesConstants.BROADCAST_FILTER.FILTERLOCATION -> locationChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERQUALITY-> qualityChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERBTHRM-> HRChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERTEMP-> tempChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERBMP-> bmpStatusChanged(intent)


                    //BROADCAST_CHANGE_TYPE_CHANGED -> handleChangeTypeChanged()
                }
            }
        }

        val manager = LocalBroadcastManager.getInstance(this)
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERLOCATION))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERQUALITY))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERBTHRM))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERTEMP))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERBMP))
        //pBLE.registerBMP()
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        if(intent.action == ServicesConstants.MOCKSERVICEACTION.STARTFOREGROUND_ACTION) {
            Log.i(LOG_TAG, "Received Start Foreground Intent ")

           // val intentNotif = Intent(this, MainActivity::class.java)
//            intentNotif.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
           // intentNotif.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACT

            //val pendingIntent = PendingIntent.getActivity(this, 0, intentNotif, 0)


            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                // Create the NotificationChannel
                val name = "ChannelName"
                val descriptionText = "ChannelDescription"
                val importance = NotificationManager.IMPORTANCE_DEFAULT
                val mChannel = NotificationChannel(CHANNEL_ID, name, importance)
                mChannel.description = descriptionText
                // Register the channel with the system; you can't change the importance
                // or other notification behaviors after this
                val notificationManager = getSystemService(NOTIFICATION_SERVICE) as NotificationManager
                notificationManager.createNotificationChannel(mChannel)
            }

            var notification = NotificationCompat.Builder(this, CHANNEL_ID)
                .setSmallIcon(R.drawable.ic_launcher_foreground)
                .setContentTitle("BLE GPS")
                .setContentText("BLE & Mock active")
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                //.setContentIntent(pendingIntent)
                .build()

            pBLE = BLEProvider(application, baseContext)
            mockGPS.start()
            startForeground(ServicesConstants.NOTIFICATION_ID.FOREGROUND_MOCK_SERVICE,notification)
        } else if(intent.action == ServicesConstants.MOCKSERVICEACTION.STOPFOREGROUND_ACTION) {
            Log.i(LOG_TAG, "Received Stop Foreground Intent ")
            pBLE.disconnectBLE()
            mockGPS.shutdown()
            stopForeground(true)
            stopSelf()
        } else if(intent.action == ServicesConstants.MOCKSERVICEACTION.TASKER_ACTION){
            Log.i(LOG_TAG, "intent action : ${intent.action}")
            taskerChanged(intent.extras)
        }
        return START_STICKY
    }
    override fun onDestroy() {
        super.onDestroy()
        Log.i(LOG_TAG, "In onDestroy")
    }

    override fun onBind(intent: Intent): IBinder? {
        Log.v(LOG_TAG, "in onBind")
        return binder
    }

    override fun onRebind(intent: Intent) {
        Log.v(LOG_TAG, "in onRebind")
        super.onRebind(intent)
    }

    override fun onUnbind(intent: Intent): Boolean {
        Log.v(LOG_TAG, "in onUnbind")
        return true
    }

    fun sendCalibrateBLE(calibrateval: Int)
    {
        val buffer = ByteBuffer.allocate(4)
        buffer.order(ByteOrder.LITTLE_ENDIAN)
        buffer.putInt(calibrateval)
        pBLE.sendCalibrate(buffer.array())
    }

    fun HRStatusChanged(enable: Boolean)
    {
        hrenable = enable
    }

    fun openGPX(datefilename: String) {
        //val datefilename: String = SimpleDateFormat("yyyyMMdd_HHmmss").format(Date())
        var gpxfilename = File(
            Environment.getExternalStorageDirectory().path + File.separator + "BTGPS",
            datefilename + ".gpx"
        )
        var gpxdir = File(Environment.getExternalStorageDirectory().path + File.separator + "BTGPS")
        var success = true;
        if (!gpxdir.exists()) {
            success = gpxdir.mkdir();
        }

        if (success) {
            if(!gpxfilename.exists()) {
                try {
                    gpxoutputstream = FileOutputStream(gpxfilename, true)
                    //val gpxheader = "GPX Header\n"
                    val gpxheader =
                        "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>\n" +
                                "<gpx xmlns=\"http://www.topografix.com/GPX/1/1\" version=\"1.1\" " +
                                "xmlns:gpxtpx=\"http://www.garmin.com/xmlschemas/TrackPointExtension/v1\" " +
                                "xmlns:blegps=\"https://www.nigreon.net\"" +
                                ">\n" +
                                "<metadata>" +
                                "<name>${datefilename}</name>" +
                                "<desc>${datefilename}</desc>\n" +
                                "<time>${SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").format(Date())}</time>\n" +
                                "</metadata>\n" +
                                "<trk>\n" +
                                "<name>${datefilename}</name>\n" +
                                "<trkseg>\n"
                    writeGPX(gpxheader)
                } catch (e: FileNotFoundException) {
                    e.printStackTrace()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            } else {
                try {
                    gpxoutputstream = FileOutputStream(gpxfilename, true)
                } catch (e: FileNotFoundException) {
                    e.printStackTrace()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
            gpxopened = true
        }
    }

    fun closeGPX()
    {
        //val gpxfooter = "GPX Footer\n"
        val gpxfooter = "</trkseg></trk></gpx>\n"
        writeGPX(gpxfooter)
        gpxoutputstream.close()
        gpxopened = false
    }

    //fun sendConfBLE(bmpnrf: Boolean, bmpble: Boolean, bmp2gps: Boolean, exttemp: Boolean, gpsble: Boolean, gpsbleverb: Boolean, gpsserial: Boolean, gpsmode: Int, gpspoller: Short, gpssendbt: Short, bmppoller: Short) {
    fun sendConfBLE(gpsConfNew: GPSConfiguration) {
        gpsConf = gpsConfNew
        val buffer = ByteBuffer.allocate(8)
        buffer.order(ByteOrder.LITTLE_ENDIAN)
        var activateByte1: Byte = 0
        var activateByte2: Byte = 0

        if(gpsConf.bmpNrf) { activateByte1 = activateByte1.or(0x01) }
        if(gpsConf.bmpBT) { activateByte1 = activateByte1.or(0x02) }
        if(gpsConf.bmp2gps) { activateByte1 = activateByte1.or(0x04) }
        if(gpsConf.extTemp) { activateByte1 = activateByte1.or(0x08) }

        if(gpsConf.gpsBLE) { activateByte2 = activateByte2.or(0x01) }
        if(gpsConf.gpsVerbose) { activateByte2 = activateByte2.or(0x02) }
        if(gpsConf.gpsSerial) { activateByte2 = activateByte2.or(0x04) }
        if(gpsConf.gpsMode == 1 || gpsConf.gpsBoost) {
            activateByte2 = activateByte2.or(0x08)
        } else if(gpsConf.gpsMode == 2) {
            activateByte2 = activateByte2.or(0x10)
        }

        buffer.put(activateByte1)
        buffer.put(activateByte2)
        if(gpsConf.gpsBoost) {
            buffer.putShort(4)
            buffer.putShort(gpsConf.bmpPollerSend)
            buffer.putShort(4)
        } else {
            buffer.putShort(gpsConf.gpsPollerSend)
            buffer.putShort(gpsConf.bmpPollerSend)
            buffer.putShort(gpsConf.gpsSendBTSend)
        }
        pBLE.sendConf(buffer.array())
    }

    inner class sBinder : Binder() {
        // Return this instance of LocalService so clients can call public methods
        fun getService(): BLEMockLocationService = this@BLEMockLocationService
    }
}