package net.nigreon.blegps
import android.app.Application
import android.bluetooth.BluetoothGatt
import android.content.Context
import android.content.Intent
import android.support.v4.content.LocalBroadcastManager
import android.util.Log
import com.clj.fastble.BleManager
import com.clj.fastble.data.BleDevice
import com.clj.fastble.exception.BleException
import com.clj.fastble.callback.BleGattCallback
import com.clj.fastble.callback.BleNotifyCallback
import com.clj.fastble.callback.BleReadCallback
import com.clj.fastble.callback.BleWriteCallback
import java.nio.ByteBuffer
import java.nio.ByteOrder
import android.os.CountDownTimer
import android.os.Handler


class BLEProvider(val application: Application, val baseContext: Context)
{
    private lateinit var bleDevice: BleDevice

    private val serviceEnvironmentalSensingUuid: String = "0000181a-0000-1000-8000-00805f9b34fb"
    private val serviceLNUuid: String = "00001819-0000-1000-8000-00805f9b34fb"
    private val characteristicPressureUuid: String = "00002a6d-0000-1000-8000-00805f9b34fb"
    private val characteristicTemperatureUuid: String = "00002a6e-0000-1000-8000-00805f9b34fb"
    private val characteristicLSUuid: String = "00002a67-0000-1000-8000-00805f9b34fb"
    private val characteristicPQUuid: String = "00002a69-0000-1000-8000-00805f9b34fb"
    private val serviceWriteUuid: String = "00002000-0000-1000-8000-00805f9b34fb"
    private val characteristicSatUuid: String = "00002002-0000-1000-8000-00805f9b34fb"
    private val characteristicWriteUuid: String = "00002001-0000-1000-8000-00805f9b34fb"
    private val characteristicCalibrateUuid: String = "00002003-0000-1000-8000-00805f9b34fb"

    private var connected = false
    private var reconnectcount = 0


    private val LOG_TAG = "BLEProvider"
    //private lateinit var baseContext: Context


    init
    {
        connectBLE()
    }

    fun delayFunction(function: ()-> Unit, delay: Long) {
        Handler().postDelayed(function, delay)
    }

    private fun connectBLE()
    {
        //BleManager.getInstance().init(application)
        /*BleManager.getInstance()
            .enableLog(true)
            .setReConnectCount(10, 60000)*/

        //BleManager.getInstance().operateTimeout = 5000
        //BleManager.getInstance().connect("24:6F:28:16:C1:F2", object : BleGattCallback() {
        BleManager.getInstance().connect("E9:8E:8A:12:6F:3F", object : BleGattCallback() {
            override fun onStartConnect() {

            }

            override fun onConnectFail(bleDevice: BleDevice, exception: BleException) {
                Log.v(LOG_TAG, "BLE Connection Failed $exception")
                if(reconnectcount <= 15) {
                    reconnectcount++
                    delayFunction( { connectBLE() }, 60000)
                } else {
                    reconnectcount=0
                    LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                        Intent(ServicesConstants.BROADCAST_FILTER.FILTERBT).putExtra(ServicesConstants.BROADCAST_KEY.KEYBTSTATUS, false)
                    )
                    connected=false
                }

            }

            override fun onConnectSuccess(bleDeviceConnect: BleDevice, gattConnect: BluetoothGatt, status: Int) {
                Log.v(LOG_TAG,  "BLE Connection OK")
                connected = true
                reconnectcount = 0
                bleDevice = bleDeviceConnect
                //initNotificationTemperature()
                object : CountDownTimer(3000, 500) {
                    var tickCount = 0
                    override fun onFinish() {
                        // When timer is finished
                        // Execute your code here
                        LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                            Intent(ServicesConstants.BROADCAST_FILTER.FILTERBT).putExtra(ServicesConstants.BROADCAST_KEY.KEYBTSTATUS, true)
                        )
                    }

                    override fun onTick(millisUntilFinished: Long) {
                        //Log.v(LOG_TAG,  "Tick: $millisUntilFinished")
                        when (tickCount) {
                            0 -> initNotificationTemperature()
                            1 -> initNotificationPQ()
                            2 -> initNotificationPressure()
                            3 -> initNotificationSat()
                            4 -> initNotificationLS()
                        }
                        tickCount++
                        /*if(millisUntilFinished == 5000L)
                        {
                            initNotificationTemperature()
                        } else if(millisUntilFinished == 4000L) {
                            initNotificationPQ()
                        } else if(millisUntilFinished == 3000L) {
                            initNotificationPressure()
                        } else if(millisUntilFinished == 2000L) {
                            initNotificationSat()
                        } else if(millisUntilFinished == 1000L) {
                            initNotificationLS()
                        }*/
                        // millisUntilFinished    The amount of time until finished.
                    }
                }.start()
                //initNotificationPressure()
                //initNotificationLS()


            }

            override fun onDisConnected(
                isActiveDisConnected: Boolean,
                bleDevice: BleDevice,
                gatt: BluetoothGatt,
                status: Int
            ) {
                //connected = false
                //LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                //    Intent(ServicesConstants.BROADCAST_FILTER.FILTERBT).putExtra(ServicesConstants.BROADCAST_KEY.KEYBTSTATUS, false)
                //)
                Log.v(LOG_TAG,  "BLE Disconnected")
                if(connected) {
                    connectBLE()
                }
            }
        })

    }

    private fun initNotificationPressure() {
        BleManager.getInstance().notify(
            bleDevice,
            serviceEnvironmentalSensingUuid,
            characteristicPressureUuid,
            object : BleNotifyCallback() {
                override fun onNotifySuccess() {
                    Log.v(LOG_TAG, "BLE Notify Pressure Success")
                    //initNotificationLS()
                }

                override fun onNotifyFailure(exception: BleException) {
                    Log.v(LOG_TAG, "BLE Notify Pressure Failure $exception")
                    //initNotificationLS()
                }

                override fun onCharacteristicChanged(data: ByteArray) {
                    val buffer = ByteBuffer.wrap(data)
                    buffer.order(ByteOrder.LITTLE_ENDIAN)
                    Log.v(LOG_TAG, "BLE Notify Pressure Characteristic changed")
                    //textpressure.text = buffer.getInt().toString()
                    LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                        Intent(ServicesConstants.BROADCAST_FILTER.FILTERPRESSURE).putExtra(ServicesConstants.BROADCAST_KEY.KEYPRESSURE, buffer.int)
                    )
                }
            })
    }

    private fun initNotificationTemperature() {
        BleManager.getInstance().notify(
            bleDevice,
            serviceEnvironmentalSensingUuid,
            characteristicTemperatureUuid,
            object : BleNotifyCallback() {
                override fun onNotifySuccess() {
                    Log.v(LOG_TAG, "Notify Temperature Success")
                    //initNotificationPressure()
                }

                override fun onNotifyFailure(exception: BleException) {
                    Log.v(LOG_TAG, "BLE Notify Temperature Failure $exception")
                    //initNotificationPressure()
                }

                override fun onCharacteristicChanged(data: ByteArray) {
                    Log.v(LOG_TAG, "Notify Temperature Characteristic changed")
                    val buffer = ByteBuffer.wrap(data)
                    buffer.order(ByteOrder.LITTLE_ENDIAN)
                    //texttemp.text = buffer.float.toString()
                    LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                        Intent(ServicesConstants.BROADCAST_FILTER.FILTERTEMP).putExtra(ServicesConstants.BROADCAST_KEY.KEYTEMP, buffer.short/100f)
                    )
                }
            })
    }

    private fun initNotificationLS() {
        BleManager.getInstance().notify(
            bleDevice,
            serviceLNUuid,
            characteristicLSUuid,
            object : BleNotifyCallback() {
                override fun onNotifySuccess() {
                    Log.v(LOG_TAG, "Notify LS Success")
                    //initNotificationPQ()
                }

                override fun onNotifyFailure(exception: BleException) {
                    Log.v(LOG_TAG, "BLE Notify LS Failure $exception")
                    //initNotificationPQ()
                }

                @UseExperimental(ExperimentalUnsignedTypes::class)
                override fun onCharacteristicChanged(data: ByteArray) {
                    Log.v(LOG_TAG, "Notify LS Characteristic changed ${data.size}")
                    val buffer = ByteBuffer.wrap(data)
                    buffer.order(ByteOrder.LITTLE_ENDIAN)
                    //val flags: UShort = buffer.short.toUShort()
                    val speed: Float = buffer.short.toUShort().toFloat()/100.0f
                    val latitude: Double = buffer.int/10000000.0
                    val longitude: Double = buffer.int/10000000.0
                    val ehpe: Float = buffer.int.toUInt().toFloat()/100.0f
                    //val ehpe: Float = 5.0f
                    val elevation: Double = buffer.int/100.0
                    val heading: Float = buffer.short.toUShort().toFloat()/100.0f
                    Log.v(LOG_TAG, "LS $latitude $longitude $speed $heading")
                    LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                        Intent(ServicesConstants.BROADCAST_FILTER.FILTERLOCATION).putExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLAT, latitude)
                                                                                 .putExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLON, longitude)
                                                                                 .putExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONEHPE, ehpe)
                                                                                 .putExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONELE, elevation)
                                                                                 .putExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONSPEED, speed)
                                                                                 .putExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONHEADING, heading)

                    )
                }
            })
    }

    private fun initNotificationPQ() {
        BleManager.getInstance().notify(
            bleDevice,
            serviceLNUuid,
            characteristicPQUuid,
            object : BleNotifyCallback() {
                override fun onNotifySuccess() {
                    Log.v(LOG_TAG, "Notify PQ Success")
                    //initNotificationSat()
                }

                override fun onNotifyFailure(exception: BleException) {
                    Log.v(LOG_TAG, "BLE Notify PQ Failure $exception")
                    //initNotificationSat()
                }

                @UseExperimental(ExperimentalUnsignedTypes::class)
                override fun onCharacteristicChanged(data: ByteArray) {
                    Log.v(LOG_TAG, "Notify PQ Characteristic changed")
                    val buffer = ByteBuffer.wrap(data)
                    buffer.order(ByteOrder.LITTLE_ENDIAN)
                    //val flags = buffer.short
                    val sat_view: Byte = buffer.get().toUByte().toByte()
                    val sat_used: Byte = buffer.get().toUByte().toByte()
                    val ehpe: Float = buffer.int.toUInt().toFloat()/100.0f
                    val evpe: Float = buffer.int.toUInt().toFloat()/100.0f
                    val hdop: Float = buffer.get().toUByte().toByte()*2/10.0f
                    val vdop: Float = buffer.get().toUByte().toByte()*2/10.0f
                    val elevation_gps: Double = buffer.int/100.0
                    Log.v(LOG_TAG, "PQ $sat_view $sat_used $ehpe $evpe $hdop $vdop $elevation_gps")
                    LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                        Intent(ServicesConstants.BROADCAST_FILTER.FILTERQUALITY).putExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYSATVIEW, sat_view)
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYSATUSED, sat_used)
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEHPE, ehpe)
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEVPE, evpe)
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYHDOP, hdop)
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYVDOP, vdop)
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYELEGPS, elevation_gps)
                    )
                }
            })
    }

    private fun initNotificationSat() {
        BleManager.getInstance().notify(
            bleDevice,
            serviceWriteUuid,
            characteristicSatUuid,
            object : BleNotifyCallback() {
                override fun onNotifySuccess() {
                    Log.v(LOG_TAG, "Notify Sat Success")
                    //initNotificationSat()
                }

                override fun onNotifyFailure(exception: BleException) {
                    Log.v(LOG_TAG, "BLE Notify Sat Failure $exception")
                    //initNotificationSat()
                }

                override fun onCharacteristicChanged(data: ByteArray) {
                    Log.v(LOG_TAG, "Notify Sat Characteristic changed")
                    val buffer = ByteBuffer.wrap(data)
                    buffer.order(ByteOrder.LITTLE_ENDIAN)
                    LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                        Intent(ServicesConstants.BROADCAST_FILTER.FILTERSAT).putExtra(ServicesConstants.BROADCAST_KEY.KEYSATGPSALL, buffer.get())
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYSATGPSUSED, buffer.get())
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYSATSBASALL, buffer.get())
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYSATSBASUSED, buffer.get())
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYSATGLONASSALL, buffer.get())
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYSATGLONASSUSED, buffer.get())
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYSATGALILEOALL, buffer.get())
                            .putExtra(ServicesConstants.BROADCAST_KEY.KEYSATGALILEOUSED, buffer.get())
                    )
                }
            })
    }

    fun disconnectBLE()
    {
        if(connected) {
        LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
            Intent(ServicesConstants.BROADCAST_FILTER.FILTERBT).putExtra(ServicesConstants.BROADCAST_KEY.KEYBTSTATUS, false)
        )
            connected = false


           BleManager.getInstance().stopNotify(
               bleDevice,
               serviceEnvironmentalSensingUuid,
               characteristicTemperatureUuid
           )
           BleManager.getInstance()
               .stopNotify(bleDevice, serviceEnvironmentalSensingUuid, characteristicPressureUuid)
           BleManager.getInstance().stopNotify(bleDevice, serviceLNUuid, characteristicLSUuid)
           BleManager.getInstance().stopNotify(bleDevice, serviceLNUuid, characteristicPQUuid)
           BleManager.getInstance().stopNotify(bleDevice, serviceWriteUuid, characteristicSatUuid)
           BleManager.getInstance().disconnect(bleDevice)
           //BleManager.getInstance().destroy()
       }
    }

    fun sendCalibrate(data: ByteArray) {
        if(connected) {
        BleManager.getInstance().write(
            bleDevice,
            serviceWriteUuid,
            characteristicCalibrateUuid,
            data,
            object : BleWriteCallback() {
                override fun onWriteSuccess(current: Int, total: Int, justWrite: ByteArray) {
                    Log.v(LOG_TAG, "Conf sended")
                }

                override fun onWriteFailure(exception: BleException) {
                    Log.v(LOG_TAG, "Conf send Failure $exception")
                }
            })
    }
    }
    fun sendConf(data: ByteArray) {
        if (connected) {
            BleManager.getInstance().write(
                bleDevice,
                serviceWriteUuid,
                characteristicWriteUuid,
                data,
                object : BleWriteCallback() {
                    override fun onWriteSuccess(current: Int, total: Int, justWrite: ByteArray) {
                        Log.v(LOG_TAG, "Conf sended")
                    }

                    override fun onWriteFailure(exception: BleException) {
                        Log.v(LOG_TAG, "Conf send Failure $exception")
                    }
                })
        }
    }
    /*fun registerBMP()
    {
        initNotificationPressure()
        initNotificationTemperature()
    }*/
}