package net.nigreon.blegps

import android.support.v7.app.AppCompatActivity
import android.os.Bundle
import kotlinx.android.synthetic.main.activity_main.*
import android.content.Intent
import android.os.IBinder
import android.content.ComponentName
import android.content.Context
import android.content.ServiceConnection
import android.support.v4.content.LocalBroadcastManager
import android.content.BroadcastReceiver
import android.content.IntentFilter
import android.os.Handler
import android.util.Log
import android.view.View
import android.widget.AdapterView
import android.location.Location
import com.clj.fastble.BleManager
import java.text.SimpleDateFormat
import java.util.*
import android.content.SharedPreferences
import android.os.Environment
import java.io.File
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException

//https://stackoverflow.com/questions/1625249/android-how-to-bind-spinner-to-custom-object-list

class MainActivity : AppCompatActivity() {

    private val LOG_TAG = "MainActivity"

    private lateinit var mBoundService: BLEMockLocationService
    var mServiceBound = false

    private lateinit var mBoundServiceHR: BLEHRService
    var mServiceBoundHR = false

    var currentPressure: Int = -1
    var currentTemperature: Float = -1.0f
    var BTConnected = false
    var BTHRConnected = false
    var lastLocation: Location = Location("LastLocation")

    private lateinit var gpxfilename: String
    private var gpxopened = false

    private lateinit var sharedPreferences: SharedPreferences


    /*override fun onStart() {
        super.onStart()
        // Bind to LocalService

    }*/

    fun delayFunction(function: ()-> Unit, delay: Long) {
        Handler().postDelayed(function, delay)
    }

    fun disconnectBT()
    {
        Intent(this, BLEMockLocationService::class.java).also { intent ->
            intent.action = ServicesConstants.MOCKSERVICEACTION.STOPFOREGROUND_ACTION
            startService(intent)
        }
        unbindService(mServiceConnection)
        mServiceBound = false
    }

    fun disconnectBTHR()
    {
        Intent(this, BLEHRService::class.java).also { intent ->
            intent.action = ServicesConstants.HRSERVICEACTION.STOPFOREGROUND_ACTION
            startService(intent)
        }
        unbindService(mServiceConnectionHR)
        mServiceBoundHR = false
    }

    fun filter01changed(intent: Intent)
    {
        Log.v(LOG_TAG, "Filter01 Broadcast received")
        text01.text=intent.getIntExtra(ServicesConstants.BROADCAST_KEY.KEY01,-1).toString()

    }
    fun temperatureChanged(intent: Intent)
    {
        Log.v(LOG_TAG, "Temperature Broadcast received")
        currentTemperature = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYTEMP, -1.0f)
        writeBMPText()
    }
    fun pressureChanged(intent: Intent)
    {
        Log.v(LOG_TAG, "Pressure Broadcast received")
        currentPressure = intent.getIntExtra(ServicesConstants.BROADCAST_KEY.KEYPRESSURE,-1)
        writeBMPText()
    }

    fun BTChanged(intent: Intent)
    {
        val connected = intent.getBooleanExtra(ServicesConstants.BROADCAST_KEY.KEYBTSTATUS,false)
        if(connected) {
            BTConnected=true
            text01.text = "BT Connected"
            Log.v(LOG_TAG, "BT Connected Broadcast received")
            mBoundService.HRStatusChanged(BTHRConnected)
            sendConfToBLEService()
            if(swlog2file.isChecked())
            {
                mBoundService.openGPX(gpxfilename)
                gpxopened = true
            }
        } else {
            text01.text = "BT Disconnected"
            Log.v(LOG_TAG, "BT Disconnected Broadcast received")
            BTConnected=false
            swblepower.setChecked(false)
        }
        writeBMPText()
    }

    fun BTChangedHR(intent: Intent)
    {
        val connected = intent.getBooleanExtra(ServicesConstants.BROADCAST_KEY.KEYBTSTATUSHR,false)
        if(connected) {
            BTHRConnected=true
            text01.text = "BT HR Connected"
            Log.v(LOG_TAG, "BT HR Connected Broadcast received")
            if(BTConnected) { mBoundService.HRStatusChanged(true) }
        } else {
            text01.text = "BT HR Disconnected"
            Log.v(LOG_TAG, "BT HR Disconnected Broadcast received")
            BTHRConnected=false
            if(BTConnected) { mBoundService.HRStatusChanged(false) }
        }
        writeBMPText()
    }

    fun BTChangedHRM(intent: Intent)
    {
        texthr.text = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYBTHRM,0).toString() + " bpm"
    }

    private fun convertLatLon(latitude: Double, longitude: Double): String {
        val builder = StringBuilder()


        val latitudeDegrees = Location.convert(Math.abs(latitude), Location.FORMAT_MINUTES)
        //val latitudeSplit1 = latitudeDegrees.split(":".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
        if (latitude < 0) {
            builder.append("S ")
        } else {
            builder.append("N ")
        }
        val latitudeSplit1 = latitudeDegrees.split(':')
        builder.append(latitudeSplit1[0])
        builder.append("°")
        val latitudeSplit2 = latitudeSplit1[1].split(',')
        builder.append(latitudeSplit2[0])
        builder.append(".")
        builder.append(latitudeSplit2[1])
        builder.append(" ")


        val longitudeDegrees = Location.convert(Math.abs(longitude), Location.FORMAT_MINUTES)
        if (longitude < 0) {
            builder.append("W ")
        } else {
            builder.append("E ")
        }
        val longitudeSplit1 = longitudeDegrees.split(':')
        builder.append(longitudeSplit1[0])
        builder.append("°")
        val longitudeSplit2 = longitudeSplit1[1].split(',')
        builder.append(longitudeSplit2[0])
        builder.append(".")
        builder.append(longitudeSplit2[1])


        return builder.toString()
    }

    fun writeBMPText()
    {
        textbmp.text = "${currentPressure/100.0f} hPa $currentTemperature°C"
    }

    fun writeLocationText(intent: Intent)
    {
        val currentLatitude = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLAT,-1.0)
        val currentLongitude = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLON,-1.0)
        val currentEHPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONEHPE,-1.0f)
        val currentElevation = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONELE,-1.0)
        val currentSpeed = (intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONSPEED,-1.0f))/1000.0f*3600.0f
        val currentHeading = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONHEADING,-1.0f)
        var currentLocation: Location = Location("CurrentLocation")
        currentLocation.latitude = currentLatitude
        currentLocation.longitude = currentLongitude

        textlastfix.text = "${SimpleDateFormat("dd-MM-yyyy HH:mm:ss.SSS").format(Date())}"
        textlocation.text = "${convertLatLon(currentLatitude, currentLongitude)} $currentElevation m \n $currentLatitude $currentLongitude $currentElevation m"
        textspeedheading.text = "$currentSpeed km/h $currentHeading° ${currentLocation.distanceTo(lastLocation)} m"
        lastLocation = currentLocation

        if(swblegpsverbose.isChecked == false) {
            textquality.text = "$currentEHPE m"
            textsatellites.text = "Satellites in verbose"
        }
    }

    fun writeQualityText(intent: Intent)
    {
        val currentEHPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEHPE,-1.0f)
        val currentEVPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEVPE,-1.0f)
        val currentHDOP = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYHDOP,-1.0f)
        val currentVDOP = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYVDOP,-1.0f)
        val sat_view = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYSATVIEW,-1)
        val sat_used = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYSATUSED,-1)
        val elevation_gps = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYELEGPS,-1.0)
        textquality.text = "$currentEHPE m $currentEVPE m $currentHDOP $currentVDOP $sat_used/$sat_view $elevation_gps m"
    }

    fun writeSatText(intent: Intent)
    {
        val gpsall = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATGPSALL,-1)
        val gpsused = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATGPSUSED,-1)
        val sbasall = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATSBASALL,-1)
        val sbasused = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATSBASUSED,-1)
        val glonassall = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATGLONASSALL,-1)
        val glonassused = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATGLONASSUSED,-1)
        val galileoall = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATGALILEOALL,-1)
        val galileoused = intent.getByteExtra(ServicesConstants.BROADCAST_KEY.KEYSATGALILEOUSED,-1)
        textsatellites.text = "GPS: $gpsused/$gpsall SBAS: $sbasused/$sbasall GLONASS: $glonassused/$glonassall GALILEO: $galileoused/$galileoall"
    }
    private fun sendCalibrateToBLEService()
    {
        if(BTConnected) {
            val calibrateval: Int = editcalibratebmp2gps.text.toString().toInt()*100
            mBoundService.sendCalibrateBLE(calibrateval)
        }
    }
    private fun sendConfToBLEService()
    {
        if(BTConnected) {
            val bmppollersend: Float  = resources.getStringArray(R.array.pollerval)[spinbmp.selectedItemPosition].toFloat() * 4
            var gpspollersend: Float
            var gpssendbtsend: Float
            var gpsmode: Int

            /*if(swblegpsboost.isChecked)
            {
                gpspollersend = 4f
                gpssendbtsend = 4f
                gpsmode = 1
            } else {*/
                gpspollersend = resources.getStringArray(R.array.pollerval)[spingps.selectedItemPosition].toFloat() * 4
                gpssendbtsend = resources.getStringArray(R.array.pollerval)[spingpssendbt.selectedItemPosition].toFloat() * 4
                gpsmode = spingpsmode.selectedItemPosition
            //}

            var gpsConf: GPSConfiguration = GPSConfiguration()

            gpsConf.bmpNrf = swbmpnrf.isChecked
            gpsConf.bmpBT = swbmpbt.isChecked
            gpsConf.bmp2gps = swbmp2gps.isChecked
            gpsConf.extTemp = swexttemp.isChecked
            gpsConf.gpsBLE = swgpsble.isChecked
            gpsConf.gpsVerbose = swblegpsverbose.isChecked
            //gpsConf.gpsBoost = swblegpsboost.isChecked
            gpsConf.gpsSerial = swgpsserial.isChecked
            gpsConf.gpsMode = gpsmode
            gpsConf.gpsPollerSend = gpspollersend.toShort()
            gpsConf.gpsSendBTSend = gpssendbtsend.toShort()
            gpsConf.bmpPollerSend = bmppollersend.toShort()

            mBoundService.sendConfBLE(gpsConf)

        }
    }
    private fun sendStopConfToBLEService()
    {
        if(BTConnected) {
            val gpspollersend: Float =
                resources.getStringArray(R.array.pollerval)[spingps.selectedItemPosition].toFloat() * 4
            val bmppollersend: Float =
                resources.getStringArray(R.array.pollerval)[spinbmp.selectedItemPosition].toFloat() * 4
            val gpssendbtsend: Float =
                resources.getStringArray(R.array.pollerval)[spingpssendbt.selectedItemPosition].toFloat() * 4

            var gpsConf: GPSConfiguration = GPSConfiguration()

            gpsConf.bmpNrf = false
            gpsConf.bmpBT = false
            gpsConf.bmp2gps = false
            gpsConf.extTemp = false
            gpsConf.gpsBLE = false
            gpsConf.gpsVerbose = false
            gpsConf.gpsSerial = false
            gpsConf.gpsMode = spingpsmode.selectedItemPosition
            gpsConf.gpsPollerSend = gpspollersend.toShort()
            gpsConf.gpsSendBTSend = gpssendbtsend.toShort()
            gpsConf.bmpPollerSend = bmppollersend.toShort()

            mBoundService.sendConfBLE(gpsConf)

        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        BleManager.getInstance().init(application)
        BleManager.getInstance()
            .enableLog(true)
            .setReConnectCount(1, 5000)

        BleManager.getInstance().operateTimeout = 5000

        val brReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context?, intent: Intent?) {

                when (intent?.action) {
                    ServicesConstants.BROADCAST_FILTER.FILTER01 -> filter01changed(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERBT -> BTChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERBTHR -> BTChangedHR(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERBTHRM -> BTChangedHRM(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERTEMP -> temperatureChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERPRESSURE -> pressureChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERLOCATION -> writeLocationText(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERQUALITY -> writeQualityText(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERSAT -> writeSatText(intent)

                    //BROADCAST_CHANGE_TYPE_CHANGED -> handleChangeTypeChanged()
                }
            }
        }

        spinbmp.setSelection(11)
        spingps.setSelection(3)
        spingpsmode.setSelection(1)
        spingpssendbt.setSelection(10)

        lastLocation.latitude = 0.0
        lastLocation.longitude = 0.0

        sharedPreferences = baseContext.getSharedPreferences("blegps_config", MODE_PRIVATE)

        /*sharedPreferences
            .edit()
            .putInt("inttest", 64)
            .putString("stringtest", "24:6F:28:16:C1:F2")
            .apply()*/

        //text01.text = "MAC ${sharedPreferences.getString("stringtest", null)}"

        //mButton.setEnabled(false)

        /*buttonread.setOnClickListener {
            //text01.text=edit01.text.toString()
            BleManager.getInstance().read(
                bleDevice,
                serviceWriteUuid,
                characteristicWriteUuid,
                object : BleReadCallback() {
                    override fun onReadSuccess(data: ByteArray) {
                        text01.text = "Read OK"
                        textread.text = data.toString()
                    }

                    override fun onReadFailure(exception: BleException) {
                        text01.text = "Read Failure $exception"
                    }
                }
            )
        }
        buttonwrite.setOnClickListener {
            val writeba: ByteArray = edit01.text.toString().toByteArray()
            BleManager.getInstance().write(
                bleDevice,
                serviceWriteUuid,
                characteristicWriteUuid,
                writeba,
                object : BleWriteCallback() {
                    override fun onWriteSuccess(current: Int, total: Int, justWrite: ByteArray) {
                        text01.text = "Write OK"
                    }

                    override fun onWriteFailure(exception: BleException) {
                        text01.text = "Write Failure $exception"
                    }
                }
            )
        }*/

        swblepower.setOnCheckedChangeListener { _, isChecked ->
            if(isChecked) {
                Intent(this, BLEMockLocationService::class.java).also { intent ->
                    bindService(intent, mServiceConnection, Context.BIND_AUTO_CREATE)
                    intent.action = ServicesConstants.MOCKSERVICEACTION.STARTFOREGROUND_ACTION
                    startService(intent)
                }
            } else {
                    sendStopConfToBLEService()
                    delayFunction({ disconnectBT() }, 1500)
            }
        }

        swblehrpower.setOnCheckedChangeListener { _, isChecked ->
            if(isChecked) {
                Intent(this, BLEHRService::class.java).also { intent ->
                    bindService(intent, mServiceConnectionHR, Context.BIND_AUTO_CREATE)
                    intent.action = ServicesConstants.HRSERVICEACTION.STARTFOREGROUND_ACTION
                    startService(intent)
                }
            } else {
                //sendStopConfToBLEService()
                delayFunction({ disconnectBTHR() }, 1500)
            }
        }

        swbmpnrf.setOnCheckedChangeListener { _,_ ->
            sendConfToBLEService()
        }
        swbmpbt.setOnCheckedChangeListener { _, checked ->
            if(checked) {
                LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                    Intent(ServicesConstants.BROADCAST_FILTER.FILTERBMP).putExtra(ServicesConstants.BROADCAST_KEY.KEYBMPSTATUS, true)
                )
            } else {
                LocalBroadcastManager.getInstance(baseContext).sendBroadcast(
                    Intent(ServicesConstants.BROADCAST_FILTER.FILTERBMP).putExtra(ServicesConstants.BROADCAST_KEY.KEYBMPSTATUS, false)
                )
            }
            sendConfToBLEService()
        }
        swbmp2gps.setOnCheckedChangeListener { _,_ ->
            sendConfToBLEService()
        }
        swexttemp.setOnCheckedChangeListener { _,_ ->
            sendConfToBLEService()
        }
        swgpsble.setOnCheckedChangeListener { _,_ ->
            sendConfToBLEService()
        }
        swblegpsverbose.setOnCheckedChangeListener { _, _ ->
            sendConfToBLEService()
        }
        swgpsserial.setOnCheckedChangeListener { _,_ ->
            sendConfToBLEService()
        }
        /*swblegpsboost.setOnCheckedChangeListener { _,_ ->
            sendConfToBLEService()
        }*/
        swlog2file.setOnCheckedChangeListener { _, checked ->
            if(checked) {
                gpxfilename = SimpleDateFormat("yyyyMMdd_HHmmss").format(Date())
                if (BTConnected) {
                    mBoundService.openGPX(gpxfilename)
                    gpxopened = true
                }
            } else {
                if(BTConnected) {
                    mBoundService.closeGPX()
                } else if(gpxopened) {
                    var gpxhandle = File(
                        Environment.getExternalStorageDirectory().path + File.separator + "BTGPS",
                        gpxfilename + ".gpx"
                    )
                    var gpxoutputstream = FileOutputStream(gpxhandle, true)
                    try {
                        val gpxfooter = "</trkseg></trk></gpx>\n"
                        gpxoutputstream.write(gpxfooter.toByteArray())
                    } catch (e: FileNotFoundException) {
                        e.printStackTrace()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }
                    gpxoutputstream.close()
                }
                gpxopened = false
            }
            //sendConfToBLEService()
        }

        spingpsmode.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(p0: AdapterView<*>?) {
                TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
            }

            override fun onItemSelected(p0: AdapterView<*>?, p1: View?, p2: Int, p3: Long) {
                sendConfToBLEService()
            }

        }

        spinbmp.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(p0: AdapterView<*>?) {
                TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
            }

            override fun onItemSelected(p0: AdapterView<*>?, p1: View?, p2: Int, p3: Long) {
                sendConfToBLEService()
            }

        }

        spingps.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(p0: AdapterView<*>?) {
                TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
            }

            override fun onItemSelected(p0: AdapterView<*>?, p1: View?, p2: Int, p3: Long) {
                sendConfToBLEService()
            }

        }
        spingpssendbt.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(p0: AdapterView<*>?) {
                TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
            }

            override fun onItemSelected(p0: AdapterView<*>?, p1: View?, p2: Int, p3: Long) {
                sendConfToBLEService()
            }

        }

        buttoncalibratebmp2gps.setOnClickListener {
            //text01.text = editcalibratebmp2gps.text
            sendCalibrateToBLEService()
        }

        /*startmock.setOnClickListener {
            //val startIntent = Intent(this,BLEMockLocationService::class.java)
            //startIntent.action = ServicesConstants.MOCKSERVICEACTION.STARTFOREGROUND_ACTION
            //startService(startIntent)
            Intent(this, BLEMockLocationService::class.java).also { intent ->
                bindService(intent, mServiceConnection, Context.BIND_AUTO_CREATE)
                intent.action = ServicesConstants.MOCKSERVICEACTION.STARTFOREGROUND_ACTION
                startService(intent)
            }
        }
        stopmock.setOnClickListener {
            //val stopIntent = Intent(this,BLEMockLocationService::class.java)
            //stopIntent.action = ServicesConstants.MOCKSERVICEACTION.STOPFOREGROUND_ACTION
            //startService(stopIntent)
            //LocalBroadcastManager.getInstance(this)
             //   .unregisterReceiver(brReceiver)
            Intent(this, BLEMockLocationService::class.java).also { intent ->
                intent.action = ServicesConstants.MOCKSERVICEACTION.STOPFOREGROUND_ACTION
                startService(intent)
            }
            unbindService(mServiceConnection)
            mServiceBound = false

        }*/
        /*bCounter.setOnClickListener {
            textread.text=mBoundService.getCounter().toString()
        }*/


        val manager = LocalBroadcastManager.getInstance(this)
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTER01))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERBT))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERBTHR))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERBTHRM))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERTEMP))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERPRESSURE))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERLOCATION))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERQUALITY))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERSAT))

        //LocalBroadcastManager.getInstance(this)
        //    .unregisterReceiver(broadCastReceiver)
    }

    override fun onPause() {
        super.onPause()
        if(!swlog2file.isChecked) {
            swblegpsverbose.setChecked(false)
        }
    }

    private val mServiceConnection = object : ServiceConnection {

        override fun onServiceDisconnected(name: ComponentName) {
            mServiceBound = false
        }

        override fun onServiceConnected(name: ComponentName, service: IBinder) {
            val binder = service as BLEMockLocationService.sBinder
            mBoundService = binder.getService()
            mServiceBound = true
        }
    }

    private val mServiceConnectionHR = object : ServiceConnection {

        override fun onServiceDisconnected(name: ComponentName) {
            mServiceBoundHR = false
        }

        override fun onServiceConnected(name: ComponentName, service: IBinder) {
            val binderHR = service as BLEHRService.sBinder
            mBoundServiceHR = binderHR.getService()
            mServiceBoundHR = true
        }
    }

}
